/* lmpconv.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include "lmp.h"

#define MAX_LONG_LENGTH 64 // Maximal length at base 2 of an integer: 64

/* convert a value to a special base, using precision */
char * lmp_conv_base (b, v, p)
unsigned b; /* output base */
long v;     /* input value */
int p;      /* output precision */
{
  static char result[MAX_LONG_LENGTH + 1];
  char c[36] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  int i = 0, sign;

  result [MAX_LONG_LENGTH] = '\0';

  sign = 0;
  if (v < 0) {
    sign = 1;
    v = -v;
  }

  b = b > 35 ? 35 : b;
  for (i = MAX_LONG_LENGTH - 1; v != 0; i--) {
    result [i] = c[v%b];
    v /= b;
  }

  if (p > -1) {
    for (;i != 0 && p >= MAX_LONG_LENGTH - i; i--)
      result[i] = '0';
    if (sign) {
      result[i] = '-';
      i--;
    }
    return &result[i+1];
  }
  else if (i != MAX_LONG_LENGTH - 1) {
    if (sign) {
      result[i] = '-';
      i--;
    }
    return &result[i+1];
  }
  else {
    result[i] = '0';
    return &result[MAX_LONG_LENGTH-1];
  }
}

/* convert a value to a special base, using precision */
char * lmp_conv_base_unsigned (b, v, p)
unsigned b;       /* output base */
unsigned long v;  /* input value */
int p;            /* output precision */
{
  static char result[MAX_LONG_LENGTH + 1];
  char c[36] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  int i = 0;

  result [MAX_LONG_LENGTH] = '\0';

  b = b > 35 ? 35 : b;
  for (i = MAX_LONG_LENGTH - 1; v != 0; i--) {
    result [i] = c[v%b];
    v /= b;
  }

  if (p > -1) {
    for (;i != 0 && p >= MAX_LONG_LENGTH - i; i--)
      result[i] = '0';
    return &result[i+1];
  }
  else if (i != MAX_LONG_LENGTH - 1) {
    return &result[i+1];
  }
  else {
    result[i] = '0';
    return &result[MAX_LONG_LENGTH-1];
  }
}
