/* lmpinfix.c
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "lmp.h"
#include "lmpmath.h"
#include "lmpinput.h"
#include <stdio.h>

//#include <stdio.h>
//#include "lmp.h"
//#include "lmpmath.h"

char * lmp_infix_op2str (lmp_operator op) {
  switch (op) {
  case LMP_OUTER_OR:
    return "||";
  case LMP_OUTER_AND:
    return "&&";
  case LMP_EQUALS:
    return "==";
  case LMP_UNEQUALS:
    return "!=";
  case LMP_GREATER_EQUAL:
    return ">=";
  case LMP_LOWER_EQUAL:
    return "<=";
  case LMP_GREATER:
    return ">";
  case LMP_LOWER:
    return "<";
  case LMP_ADDITION:
    return "+";
  case LMP_SUBTRACTION:
    return "-";
  case LMP_MULTIPLICATION:
    return "*";
  case LMP_DIVISION:
    return "/";
  case LMP_MODULO:
    return "mod";
  case LMP_DIV_NO_REM:
    return "div";
  case LMP_POWER:
    return "^";
  case LMP_LOG:
    return "log";
  case LMP_SINE:
    return "sin";
  case LMP_COSINE:
    return "cos";
  case LMP_TANGENT:
    return "tan";
  case LMP_COTANGENT:
    return "cot";
  case LMP_ARC_SINE:
    return "asin";
  case LMP_ARC_COSINE:
    return "acos";
  case LMP_ARC_TANGENT:
    return "atan";
  case LMP_LOGARITHM:
    return "ln";
  case LMP_LOG_10:
    return "lg";
  case LMP_EXP:
    return "exp";
  case LMP_SQRT:
    return "sqrt";
  case LMP_UNARY_MINUS:
    return "-";
  }

  return NULL;
}

char * lmp_infix_ctrl2str (lmp_control op) {
  switch (op) {
  case LMP_DIFF:
    return "diff";
  case LMP_INT:
    return "int";
  case LMP_SUM:
    return "sum";
  case LMP_PROD:
    return "prod";
  }
  return NULL;
}

char * lmp_infix_from_tree (lmp_node_t * root, int expand) {
  lmp_node_t * p;
  char * res;

  p = root;
  if (p) {
    switch (p->what) {
    case OP:
      switch (p->data.operation.op) {
      case LMP_OUTER_OR:
      case LMP_OUTER_AND:
      case LMP_EQUALS:
      case LMP_UNEQUALS:
      case LMP_GREATER_EQUAL:
      case LMP_LOWER_EQUAL:
      case LMP_GREATER:
      case LMP_LOWER:
      case LMP_ADDITION:
      case LMP_SUBTRACTION:
      case LMP_MULTIPLICATION:
      case LMP_DIVISION:
      case LMP_MODULO:
      case LMP_DIV_NO_REM:
      case LMP_POWER:
	asprintf (&res, "(%s %s %s)",
		  lmp_infix_from_tree (p->data.operation.argv[0], expand),
		  lmp_infix_op2str (p->data.operation.op),
		  lmp_infix_from_tree (p->data.operation.argv[1], expand));
	break;
      case LMP_LOG:
	asprintf (&res, "(%s %s %s)",
		  lmp_infix_op2str (p->data.operation.op),
		  lmp_infix_from_tree (p->data.operation.argv[0], expand),
		  lmp_infix_from_tree (p->data.operation.argv[1], expand));
	break;
      case LMP_SINE:
      case LMP_COSINE:
      case LMP_TANGENT:
      case LMP_COTANGENT:
      case LMP_ARC_SINE:
      case LMP_ARC_COSINE:
      case LMP_ARC_TANGENT:
      case LMP_LOGARITHM:
      case LMP_LOG_10:
      case LMP_EXP:
      case LMP_SQRT:
      case LMP_UNARY_MINUS:
	asprintf (&res, "(%s %s)",
		  lmp_infix_op2str (p->data.operation.op),
		  lmp_infix_from_tree (p->data.operation.argv[0], expand));
	break;
      }
      break;
    case CONTROL:
      if (expand) {
	res = lmp_infix_from_tree (lmp_proc_control (p), expand);
      }
      else
	switch (p->data.control.op) {
	case LMP_DIFF:
	case LMP_INT:
	  asprintf (&res, "(%s %s %s)",
		    lmp_infix_ctrl2str (p->data.control.op),
		    lmp_infix_from_tree (p->data.control.argv [0], expand),
		    lmp_infix_from_tree (p->data.control.argv [1], expand));
	  break;
	case LMP_SUM:
	case LMP_PROD:
	  asprintf (&res, "(%s %s %s %s %s)",
		    lmp_infix_ctrl2str (p->data.control.op),
		    lmp_infix_from_tree (p->data.control.argv [0], expand),
		    lmp_infix_from_tree (p->data.control.argv [1], expand),
		    lmp_infix_from_tree (p->data.control.argv [2], expand),
		    lmp_infix_from_tree (p->data.control.argv [3], expand));
	  break;
	}
      break;
    case VAL:
      //      asprintf (&res, "{%" FREAL ",%" FREAL "}", CREAL (p->data.value), CIMAG (p->data.value));
      if (IS_REAL (p->data.value))
	res = lmp_print_number_to_string (CREAL (p->data.value));
      else
	asprintf (&res, "{%s,%s}",
		  lmp_print_number_to_string (CREAL (p->data.value)),
		  lmp_print_number_to_string (CIMAG (p->data.value)));
      break;
    case VAR:
      asprintf (&res, "%s", p->data.variable);
      break;
    }
  }

  return res;
}
