/* lmpmath.h
 *
 * Copyright (C) 1999-2000 Bernhard Pietsch <bjtp@gmx.net>
 *
 * This file is part of LMP library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __LMPMATH_H
#define __LMPMATH_H

#include <math.h>
#include <lmp.h>

#ifdef HAVE_LONG_DOUBLE

# ifdef M_LN10l
#  define LMP_CONST_LN10 M_LN10l
# else
#  define LMP_CONST_LN10 2.3025850929940456840179914546843642L
# endif

# ifdef HAVE_POWL
#  define POW(n,e)     powl (n, e)
# endif
# ifdef HAVE_FMODL
#  define MOD(a,b)     fmodl (a, b)
# endif
# ifdef HAVE_SINL
#  define SIN(a)       sinl (a)
# endif
# ifdef HAVE_COSL
#  define COS(a)       cosl (a)
# endif
# ifdef HAVE_TANL
#  define TAN(a)       tanl (a)
# endif
# ifdef HAVE_ASINL
#  define ARCSIN(a)    asinl (a)
# endif
# ifdef HAVE_ACOSL
#  define ARCCOS(a)    acosl (a)
# endif
# ifdef HAVE_ATANL
#  define ARCTAN(a)    atanl (a)
# endif
# ifdef HAVE_LOGL
#  define LOGN(a)      logl (a)
# endif
# ifdef HAVE_LOG10L
#  define LOG10(a)    log10l (a)
# endif
# ifdef HAVE_EXPL
#  define EXP(a)      expl (a)
# endif
# ifdef HAVE_SQRTL
#  define SQRT(a)     sqrtl (a)
# endif

# ifdef HAVE_MODFL
#  define MODF(a,b)   modfl (a, b)
# endif
# ifdef HAVE_FABSL
#  define ABS(a)      fabsl (a)
# endif
# ifdef HAVE_FINITEL
#  define FINITE(a)   finitel (a)
# endif

# ifdef HAVE_COMPLEX
#  ifdef HAVE_POWL
#   define CPOW(n,e)    cpowl (n, e)
#  endif
#  ifdef HAVE_SINL
#   define CSIN(a)      csinl (a)
#  endif
#  ifdef HAVE_COSL
#   define CCOS(a)      ccosl (a)
#  endif
#  ifdef HAVE_TANL
#   define CTAN(a)      ctanl (a)
#  endif
#  ifdef HAVE_ASINL
#   define CARCSIN(a)   casinl (a)
#  endif
#  ifdef HAVE_ACOSL
#   define CARCCOS(a)   cacosl (a)
#  endif
#  ifdef HAVE_ATANL
#   define CARCTAN(a)   catanl (a)
#  endif
#  ifdef HAVE_LOGL
#   define CLOGN(a)     clogl (a)
#  endif
#  ifdef HAVE_LOG10L
#   define CLOG10(a)    clog10l (a)
#  endif
#  ifdef HAVE_EXPL
#   define CEXP(a)      cexpl (a)
#  endif
#  ifdef HAVE_SQRTL
#   define CSQRT(a)     csqrtl (a)
#  endif
#  ifdef HAVE_CREALL
#   define CREAL(a)     creall (a)
#  endif
#  ifdef HAVE_CIMAGL
#   define CIMAG(a)     cimagl (a)
#  endif
# else
#  ifndef CREAL
#   define CREAL(a)    (a)
#  endif
#  ifndef CIMAG
#   define CIMAG(a)    (0.0l)
#  endif
# endif
#endif


#ifndef LMP_CONST_LN10
# ifdef M_LN10
#  define LMP_CONST_LN10 M_LN10
# else
#  define LMP_CONST_LN10 2.30258509299404568402
# endif
#endif

#ifndef POW
# define POW(n,e)    pow (n, e)
#endif
#ifndef MOD
# define MOD(a,b)    fmod (a, b)
#endif
#ifndef SIN
# define SIN(a)      sin (a)
#endif
#ifndef COS
# define COS(a)      cos (a)
#endif
#ifndef TAN
# define TAN(a)      tan (a)
#endif
#ifndef ARCSIN
# define ARCSIN(a)   asin (a)
#endif
#ifndef ARCCOS
# define ARCCOS(a)   acos (a)
#endif
#ifndef ARCTAN
# define ARCTAN(a)   atan (a)
#endif
#ifndef LOGN
# define LOGN(a)     log (a)
#endif
#ifndef LOG10
# define LOG10(a)    log10 (a)
#endif
#ifndef EXP
# define EXP(a)      exp (a)
#endif
#ifndef SQRT
# define SQRT(a)     sqrt (a)
#endif

#ifndef MODF
# define MODF(a,b)   modf (a, b)
#endif
#ifndef ABS
# define ABS(a)      fabs (a)
#endif
#ifndef FINITE
# ifdef HAVE_FINITE
#  define FINITE(a)   finite (a)
# else
#  define FINITE(a)   _finite (a)
# endif
#endif

#ifdef HAVE_COMPLEX
# ifndef CPOW
#  define CPOW(n,e)    cpow (n, e)
# endif
# ifndef CSIN
#  define CSIN(a)      csin (a)
# endif
# ifndef CCOS
#  define CCOS(a)      ccos (a)
# endif
# ifndef CTAN
#  define CTAN(a)      ctan (a)
# endif
# ifndef CARCSIN
#  define CARCSIN(a)   casin (a)
# endif
# ifndef CARCCOS
#  define CARCCOS(a)   cacos (a)
# endif
# ifndef CARCTAN
#  define CARCTAN(a)   catan (a)
# endif
# ifndef CLOGN
#  define CLOGN(a)     clog (a)
# endif
# ifndef CLOG10
#  define CLOG10(a)    clog10 (a)
# endif
# ifndef CEXP
#  define CEXP(a)      cexp (a)
# endif
# ifndef CSQRT
#  define CSQRT(a)     csqrt (a)
# endif
# ifndef CREAL
#  define CREAL(a)     creal (a)
# endif
# ifndef CIMAG
#  define CIMAG(a)     cimag (a)
# endif
# define IS_REAL(a)   (CIMAG (a) == 0.0l)
#else
# ifndef CREAL
#  define CREAL(a)    (a)
# endif
# ifndef CIMAG
#  define CIMAG(a)    (0.0)
# endif
# define IS_REAL(a)   (1)
#endif

#define IS_INT(a) ((lmp_value_type) ((long long) a) == a)


#ifdef _STDARG_FP_SAFE
# define NEW_VAL(a) lmp_node_new (VAL, a)
#else
# define NEW_VAL(a) lmp_node_new_fp_safe (VAL, a)
#endif
#endif
